import com.jinsight.jetchart.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.text.DecimalFormat;
import java.text.NumberFormat;

/**
 * This example demonstrates how to handle series events.
 *
 * To  test  click  events,  click   each   data point(a bar) of  a 
 * StackBarSerie  object  or  BarSerie  object and watch the output
 * in the console. 
 *
 * To  test  dragging  events,  move the mouse cursor to the top of
 * any   bar  and  when the cursor shape changes, click and drag. A
 * SerieEvent  is   dispatched  to the serieDragged(SerieEvent evt)
 * method. The  value  of the  respective element is adjusted while
 * a bar is resized.
 *
 * To test mouse  moving  events, move mouse cursor on any bar. The 
 * top panel displays  the  data points  values. If   tooltips  are
 * not  enabled  and  chart  is  2D,  series events  are dispatched
 * to the mouseMoved(SerieEvent evt) method. Click the checkbox  in
 * the top panel to enable/disable tooltips.
 *
 * @author Andre de Lima Soares - alsoares@jinsight.com
 */
public class Main extends JFrame implements SerieListener,ItemListener {

    BarSerie b;
    StackBarSerie sb,sb2;
    Graph graph;
    JLabel l1,l2,l3;
    NumberFormat nf;
    DecimalFormat dec;
    JCheckBox showToolTip;
    JPanel topPanel;

    public Main() {

        setTitle("The JetChart Library - Series event demo");

        addWindowListener(new WindowAdapter() {
           public void windowClosing(WindowEvent evt) {
              System.exit(0);
           }
        });
	
	// Creates the top panel.
	createTopPanel();

        Container ct=getContentPane();

	ct.add("North",topPanel);

	// Gets a NumberFormat instance to format values. Casts it to 
	// a DecimalFormat, if possible. 
	nf=NumberFormat.getInstance();
	if (nf instanceof DecimalFormat) {
	    dec=(DecimalFormat)nf;
	    dec.applyPattern("###,##0.00");
	}

	graph=new Graph();

	graph.setLabels(new String[]{"label1","label2","label3","label4","label5",
				     "label6","label7","label8","label9","label10"});
	
        graph.setTitle(new String[]{"The JetChart Library - Series events demo"});
	graph.setGradientColors(Color.yellow,new Color(0,140,255));

	// Enables the grid of the primary GraphSet object.
	// See documentation of the GraphSet class for further details.
	graph.getGraphSet(0).getGrid().setEnabled(true);

	// The Main object is added as a listener and must implement the SerieListener interface.
	graph.addSerieListener(this);
	
	// Creates a bar series
	b=new BarSerie(new double[]{200,210,140,430,320,500,340,450,100,133},"Bar Series");
	b.setColor(Color.blue);

	// Creates two stacked bars series
	sb=new StackBarSerie(new double[]{143,100,210,345,400,320,240,400,430,500},"Stacked bar 1");
	sb.setColor(Color.yellow);

	sb2=new StackBarSerie(new double[]{320,110,400,500,300,200,100,140,230,112},"Stacked bar 2");
	sb2.setColor(Color.orange);

	// Sets width of stacked bars to 15.
	StackBarSerie.setStackBarWidth(15);

	graph.addSerie(b);
	graph.addSerie(sb);
	graph.addSerie(sb2);

	// Enables series dragging.
	// To drag a series, move the mouse cursor over the top  of  a bar or a coordinate of
	// a line series, area series, etc. When the cursor changes to the resizing shape, click and
	// drag. While dragging, a SerieEvent object is dispatched to the serieDragged(SerieEvent evt)
	// method of all SerieListeners registered with the Graph object. See documentation of 
	// SerieListener and SerieEvent.
	graph.setSerieDraggingEnabled(true);

	ct.add("Center",graph);

        setSize(550,450);

        setVisible(true);

    }

    public void itemStateChanged(ItemEvent evt) {

	// Enables/disables tooltip.  If  tooltips  display  is enabled,  series   events 
	// are not dispatched to the mouseMoved method of registered SerieListener objects. 
	graph.getToolTip().setEnabled(showToolTip.isSelected());
    }

    // When mouse is released, a SerieEvent is created and dispatched to the serieReleased
    // method of registered SerieListener objects. 
    // When mouse button is released after any bar is clicked or dragged the top labels
    // displaying values are emptied.
    public void serieReleased(SerieEvent evt) {
	l1.setText("");
	l2.setText("");
	l3.setText("");
	
    }

    // A mouse click on any bar displays bar value in the respective top label. If a legend 
    // symbol is clicked a message is outputted to the console.
    public void serieClicked(SerieEvent evt) {
	
	// gets the index of the series element(a data point or a legend icon).
	int elementIndex=evt.getElementIndex();
	
	// if legend was not clicked...
	if (!evt.isOverLegend()) {
	    if (evt.getSerie()==b)
		System.out.println("Element nr "+elementIndex+" of series named Bar Series was clicked");
	    else if (evt.getSerie()==sb)
		System.out.println("Element nr "+elementIndex+" of series named Stacked bar 1 was clicked");
	    else if (evt.getSerie()==sb2)
		System.out.println("Element nr "+elementIndex+" of series named Stacked bar 2 was clicked");
	}
	// A legend symbol is clicked.
	else {
	    AbstractSerie as=evt.getSerie();
	    String seriesTitle=((GraphSerie)as).getTitle();
	    System.out.println("Legend symbol of series named "+seriesTitle+" was clicked");
	}


    }
 
    /**
     * Line series, point series,image series, bar series, stacked bar series and area series 
     * can be dragged. A pie series can't be dragged. 
     * When cursor is moved to the top of any bar its shape is changed to a resizing cursor,
     * indicating that a bar can be resized. Click mouse button and drag the top of a bar. While
     * dragging the value associated with a bar is recalculated to reflect the change in 
     * size and the new value is then displayed in the top panel.
     */
    public void serieDragged(SerieEvent evt) {
	
	int elementIndex=evt.getElementIndex();
	double[] values=evt.getSerie().getValues();
	AbstractSerie as=evt.getSerie();

	String seriesTitle=((GraphSerie)as).getTitle();

	String newValue=dec.format(values[elementIndex]);

	if (as==b)
	    l1.setText(newValue);
	else if (as==sb)
	    l2.setText(newValue);
	else 
	    l3.setText(newValue);
	
    }

    // If mouse cursor is moved over a data point, chart is 2D and tooltips display is disabled,
    // a series event is dispatched to the mouseMoved method of registered SerieListener objects.
    // Data points values are retrieved and displayed in the top panel.
    public void mouseMoved(SerieEvent evt) {

	AbstractSerie as=evt.getSerie();
	int elementIndex=evt.getElementIndex();
	String text="";
	
	// Checks the data point index. If it is not -1, gets 
	// respective series values and formats it.
	if (elementIndex!=-1) {
	    double value=as.getValues()[elementIndex];
	    if (dec!=null)
		text=dec.format(value);
	    else
		text=Double.toString(value);
	}
	
	// Displays data points values as a string.
	if (as==b) 
	    l1.setText(text);
	else if (as==sb)
	    l2.setText(text);
	else if (as==sb2)
	    l3.setText(text);
    }
    

    private void createTopPanel() {
	topPanel=new JPanel();
        topPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

	l1=createLabel();
	l2=createLabel();
	l3=createLabel();
	
	showToolTip=new JCheckBox("Show tooltips",false);
	showToolTip.addItemListener(this);

	topPanel.add(showToolTip);
	
        JLabel label=new JLabel("Bar:");
	topPanel.add(label);
	topPanel.add(l1);

        label=new JLabel("Stacked bar 1:");
	topPanel.add(label);
	topPanel.add(l2);

        label=new JLabel("Stacked bar 2:");
	topPanel.add(label);
	topPanel.add(l3);

    }

    private JLabel createLabel() {
	JLabel l=new JLabel();
        l.setPreferredSize(new Dimension(50,24));
	return l;
    }

    public static void main(String[] args) {
        new Main();
    }
}
