import java.awt.*;
import java.applet.*;
import com.jinsight.jetchart.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

/**
 * This application  demonstrates how  to  combine  graph  zooming  with  scrolling  of
 * zoomed areas. JetChart  dispatches  a  zooming event  each  time  chart is zoomed-in
 * or zoomed-out. By capturing these events the application is able to set the properties
 * of a scrolling component, making it possible to adjust the set of visible data points 
 * while sliding the scrolling bar. The flexibility  achieved with this simple model is
 * far better  than having  the  scrolling mechanism  implemented in the class library,
 * since any scrolling component can be used.
 * In order to capture zoom events, the  application  must adhere to  the ZoomListener
 * interface, implementing only one method(ZoomListener.chartZoomed(ZoomEvent evt)).
 *
 * @author Andre de Lima Soares - alsoares@jinsight.com
 */
public class Main extends JFrame implements ZoomListener,AdjustmentListener {

    Graph graph;

    OHLCSerie os;

    JScrollBar scrollBar;

    public Main() {
        
	setTitle("The JetChart Library - zoom demo");

	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent evt) {
		    System.exit(0);
		}
	    });

        // Creates a scrolling bar and adds it to a bottom panel.
	JPanel bottomPanel=new JPanel(new BorderLayout());
	scrollBar=new JScrollBar(JScrollBar.HORIZONTAL,0,0,0,0);
	scrollBar.addAdjustmentListener(this);
	bottomPanel.add("Center",scrollBar);

        Container ct=getContentPane();
	ct.add("South",bottomPanel);
	
	// Creates the chart context and sets several properties.
	createGraph();

        // Zoom events are captured by this application.
	graph.addZoomListener(this);

	/**
	 * Sets the properties  of  the primary GraphSet. JetChart  holds  a primary
         * GraphSet object. At initialization time, all GraphSet objects are verified
	 * to  detect series  that were not added to them. The series detected  are
	 * then added to the primary GraphSet.
	 */
	setPrimaryGraphSetProperties();
	
	createSeries();

	ct.add(BorderLayout.CENTER,graph);

	setSize(500,400);

	setVisible(true);

    }

    // Implements the ZoomListener interface. Each time chart is zoomed,
    // a ZoomEvent object is dispatched to this method.
    public void chartZoomed(ZoomEvent evt) {

         if (evt.getZoomType()==ZoomEvent.ZOOM_IN) { // If chart is zoomed-in...
            scrollBar.setMaximum(evt.getMaximum());
            scrollBar.setVisibleAmount(evt.getVisibleAmount());
            scrollBar.setValue(evt.getValue());
        }
        else if (evt.getZoomType()==ZoomEvent.ZOOM_OUT) { // If chart is zoomed-out... 
            scrollBar.setMaximum(0);
            scrollBar.setVisibleAmount(0);
            scrollBar.setValue(0);
        }

    }

    // Implements the AdjustmentListener interface. While sliding a scrolling
    // bar, an  AdjustmentEvent  is  dispatched to this method. The scrolling
    // bar properties are captured and assigned to the setDataSetRange method.
    public void adjustmentValueChanged(AdjustmentEvent evt) {

	int startIndex=evt.getValue();
	int endIndex=evt.getValue()+scrollBar.getVisibleAmount()-1;

        // This  method  is used  to shrink the  range  of visible data points,
        // by setting the first and last range indexes. Any data points outside
        // of this range are not displayed.
	graph.setDataSetRange(startIndex,endIndex);

	graph.repaint();

    }

    private void createGraph() {

	graph=new Graph();
	graph.setTitle(new String[]{"The JetChart Library - A financial chart"});

	// Sets Legend properties
	graph.getLegend().setOpacityEnabled(false);

	// Sets a start date. 
	graph.setStartDate("01272001");

	// Enables month labels.
	graph.setMonthLabelsEnabled(true);
	
	// Enables zoom. To select an area to be zoomed-in, click mouse left button and drag. After finishing
	// selection, just release button. The selected area is zoomed-in. 
	// To return chart to its original state, click mouse right button.
	// If chart dragging is enabled, this method has no effect.
	graph.setZoomEnabled(true);

        // Sets tooltips properties.
        setToolTipProperties();
       
	/**
	 * An OHLC chart can display a scrolling   label.
	 * The scrolling label is only displayed if chart
	 * is 2D and orientation is vertical.
	 */
	graph.getScrollingLabel().setEnabled(true);

	// Sets scrolling label description.
	graph.getScrollingLabel().setDescription("Date:");

	// Sets scrolling label color.
	graph.getScrollingLabel().setColor(Color.red);

	// Sets chart background color.
	graph.setBackground(new Color(204,210,211));

        
    }

    private void setToolTipProperties() {

       	// Enables simultaneous display of tooltips.
	graph.getToolTip().setMultipleEnabled(true);

        /**
	 * Sets the tooltip border color to be the same of its respective series.
	 * A series can be identified by checking its tooltip border color. 
         */
	graph.getToolTip().setMatchingBorderEnabled(true);

        /**
         * The tooltips matching borders are only displayed if the tooltip type
         * is set to NONE.
         */
        graph.getToolTip().setPointerType(ToolTip.NONE);

	// Sets tooltips background and font.
	graph.getToolTip().setBackground(Color.white);

    }

    private void setPrimaryGraphSetProperties() {
	
	/**
	 * Gets a reference to the primary GraphSet object. Each GraphSet object
	 * included in the chart   context  has  an  index. The primary GraphSet
	 * has index 0.
	 */
	GraphSet graphSet=graph.getGraphSet(0);
	
        // Sets the scale properties of the primary GraphSet.
	Scale scale=graphSet.getScale();
	
	scale.setPosition(Scale.RIGHT);

	/**
	 * Disables automatic scale and sets the maximum, minimum and increment
	 * values.
	 */
	scale.setAutoScaleEnabled(false);
	scale.setMaxValue(220);
	scale.setMinValue(30);
	scale.setIncrement(30);

        // Sets the scale value format.
	scale.setValueFormat("$ ###,###0.00");

	// Disables scale ticks.
	scale.setMarksEnabled(false);

        // Enables the grid.
	graphSet.getGrid().setEnabled(true);
	graphSet.getGrid().setColor(Color.orange);

	/**
	 * Enables the scrolling bar. A scrolling bar is displayed   across
	 * the chart area while moving mouse. It is only displayed if chart
	 * is 2D.
	 */
	ScrollingBar scrollingBar=graphSet.getScrollingBar();
	scrollingBar.setEnabled(true);

	// Enables the grid crossed lines and sets its color to light gray.
	GridCrossedLines gridCrossedLines=graphSet.getGridCrossedLines();
	gridCrossedLines.setEnabled(true);
	gridCrossedLines.setColor(Color.lightGray);
	
	// Enables the chart wall and sets its color.
	Wall wall=graphSet.getWall();
	wall.setEnabled(true);
	wall.setColor(new Color(255,255,214));

	// Enables wall shadow.
	wall.setShadowEnabled(true);

	// Disables labels ticks.
	graphSet.setLabelsMarksEnabled(false);

        // Disables the left and right vertical axis.
        graphSet.getYAxis().setEnabled(false);
        graphSet.getRYAxis().setEnabled(false);


    }

    private void createSeries() {

	double[][] seriesValues=getSeriesValues();
	
  	 // Creates OHLC series.
	os=new OHLCBarSerie(seriesValues);
	os.setType(OHLCBarSerie.BAR_OHLC);
	((OHLCBarSerie)os).setWidth(2);
	
	// Sets the bullish and bearish colors of the OHLC bar series.
	((OHLCBarSerie)os).setBullishColor(Color.green.darker());
	((OHLCBarSerie)os).setBearishColor(Color.red);
	
	((OHLCBarSerie)os).setDashSize(1);

	os.setTitle("Stock prices");
	os.setColor(Color.green.darker());

        /**
	 * Adds  the  stock prices series to the chart context. This  series  is  automatically
	 * added to the primary GraphSet object, since it wasn't added to a secondary graph set.
         */
	graph.addSerie(os);

    }
    
     private double[][] getSeriesValues() {
	    
	double[][] values={{185.00,178.00,179.00,182.50},{196.00,183.50,183.95,192.90},
			   {196.95,189.20,194.00,190.00},{195.50,186.55,191.00,187.25},
			   {195.60,190.00,192.00,192.00},{207.35,190.10,191.10,207.35},
			   {214.00,193.10,211.25,196.15},{202.80,189.50,196.00,196.45},
			   {206.35,194.30,197.00,199.00},{208.00,197.00,204.00,198.10},
			   {205.00,182.30,199.00,182.95},{189.70,182.80,184.00,184.50},
			   {188.00,180.00,185.00,182.60},{184.45,173.15,184.00,174.10},
			   {180.00,173.00,178.00,174.25},{178.00,167.05,175.00,169.00},
			   {173.80,159.10,170.00,160.90},{165.50,160.00,163.00,163.75},
			   {173.00,161.10,163.00,163.20},{174.50,167.00,168.00,174.10},
			   {179.50,160.20,174.60,160.20},{163.90,154.00,160.10,161.30},
			   {163.20,148.40,161.95,148.40},{146.50,136.55,144.60,136.55},
			   {147.45,146.10,147.45,147.45},{159.20,141.50,155.10,154.00},
			   {153.00,142.10,153.00,143.10},{146.00,136.00,146.00,138.50},
			   {139.10,130.00,139.00,131.35},{137.70,125.75,134.00,128.45},
			   {138.25,127.00,128.50,134.90},{144.20,134.00,134.00,135.70},
			   {136.30,128.00,134.50,128.00},{132.50,126.30,132.00,128.85},
			   {139.10,129.40,129.40,139.10},{143.00,135.00,141.90,135.00},
			   {137.00,132.10,136.75,133.70},{139.90,130.25,137.00,131.85},
			   {137.50,127.00,130.95,137.40},{138.40,129.05,137.80,130.45},
			   {132.85,127.00,130.00,131.50},{142.00,128.00,129.15,140.40},
			   {151.60,140.00,141.00,151.60},{159.85,139.50,155.00,143.95},
			   {155.45,140.00,145.80,155.45},{163.90,150.00,161.40,154.75},
			   {167.10,155.90,156.50,167.10},{175.00,164.00,169.80,166.00},
			   {165.00,157.05,165.00,160.00},{162.00,154.20,159.00,154.80},
			   {154.00,143.75,154.00,145.25},{156.85,134.10,135.10,155.00},
			   {160.00,149.00,155.25,152.00},{156.00,144.65,153.50,148.50},
			   {148.00,139.00,148.00,143.85},{152.85,143.50,149.90,144.95},
			   {144.20,135.00,144.20,139.90},{150.75,139.90,142.00,149.00},
			   {149.00,140.55,149.00,142.70},{143.50,138.60,143.45,139.00},
			   {144.80,128.00,142.15,128.70},{131.45,124.50,128.00,129.70},
			   {130.25,122.60,129.00,125.85},{128.40,120.00,126.50,124.10},
			   {127.30,122.65,125.00,123.00},{128.95,115.50,123.00,126.95},
			   {128.85,120.10,127.00,120.65},{122.85,116.00,120.00,120.20},
			   {122.00,111.75,120.50,112.60},{118.35,108.50,118.35,111.50},
			   {114.40,110.25,112.00,110.50},{113.90,109.10,112.00,110.80},
			   {112.50,107.20,111.00,108.60},{108.50,105.75,107.35,106.20},
			   {107.35,95.50,105.00,96.80},{102.50,96.00,97.50,99.50},
			   {108.00,97.00,99.00,106.45},{115.50,106.75,106.90,114.40},
			   {126.00,113.00,114.00,125.50},{130.00,123.00,128.40,124.00},
			   {125.00,118.50,125.00,120.90},{123.85,112.20,122.00,113.50},
			   {113.20,103.00,112.25,109.20},{112.00,107.00,107.00,110.70},
			   {117.95,112.60,113.00,113.05},{114.35,110.00,114.00,112.00},
			   {114.45,111.20,111.70,112.30},{118.00,112.20,113.00,117.15},
			   {122.20,116.50,118.00,117.65},{123.40,118.55,120.00,119.65},
			   {122.00,118.30,119.00,120.30},{126.80,120.50,120.50,123.10},
			   {124.80,121.15,123.30,121.60},{123.00,119.10,121.50,122.70},
			   {126.70,121.50,124.95,122.50},{125.00,118.00,123.90,119.55},
			   {120.80,115.50,119.90,115.90},{117.30,114.00,116.40,116.80},
			   {119.50,115.10,116.00,116.00},{117.75,114.00,114.60,116.20},
			   {121.00,116.00,116.60,118.15},{120.00,117.30,118.85,117.95},
			   {123.50,118.20,118.75,122.00},{122.80,117.50,122.10,117.95},
			   {125.00,118.60,120.00,120.70},{128.00,120.75,121.10,127.00},
			   {133.25,127.00,127.75,127.30},{128.70,124.80,126.15,125.20},
			   {131.25,125.00,125.90,129.45},{134.40,128.70,131.80,129.85},
			   {132.15,128.00,130.40,130.35},{136.80,130.05,130.90,136.15},
			   {138.40,132.00,136.90,132.25},{135.85,130.60,133.00,134.00},
			   {139.90,134.50,136.00,139.30},{140.90,136.40,139.00,137.55},
			   {137.50,129.55,136.50,132.10},{133.60,123.05,132.30,126.10},
			   {128.50,120.60,127.20,121.35},{120.00,107.00,117.00,110.10},
			   {117.40,106.10,109.00,114.85},{119.00,106.25,116.00,107.45},
			   {109.80,103.00,106.20,107.85},{109.50,104.35,107.50,106.50},
			   {110.05,105.10,105.10,108.00},{115.40,104.00,107.35,113.05},
			   {118.85,110.10,112.50,116.70},{119.40,112.25,117.50,112.80},
			   {116.00,111.55,113.70,114.65},{115.75,107.50,114.40,109.05},
			   {115.90,106.15,109.70,115.20},{119.50,115.50,115.90,117.40},
			   {117.90,114.00,117.00,115.95},{121.00,113.25,115.00,120.65},
			   {126.00,119.75,121.25,125.00},{126.90,120.10,126.00,121.45},
			   {121.30,112.20,121.00,113.15},{115.25,113.05,113.90,113.95},
			   {117.00,114.10,115.50,115.70},{117.80,115.20,115.95,116.40},
			   {119.80,116.00,116.25,116.65},{117.70,113.00,117.40,116.10},
			   {124.80,115.00,116.50,121.80},{128.10,122.10,123.25,124.70},
			   {126.50,122.70,124.10,124.20},{125.45,121.60,125.00,123.30},
			   {126.75,122.50,123.80,124.50},{127.50,122.10,126.00,123.55},
			   {125.50,121.35,122.30,121.55},{121.20,117.55,121.20,118.40}};

	return values;

    }

    public static void main(String[] args) {
	Main m=new Main();
    }

}

