import java.io.*;
import java.util.Date;

/**
 * FileCheckThread.java
 *
 * @author Andre de Lima Soares - alsoares@jinsight.com
 * @version 1.1
 */
public class FileCheckThread extends Thread {
    
    private String writeDir,fileExtension;
    private int expiryTime,checkTime;
    
    boolean stop;

    /**
     * The thread has four arguments used to control file deletion:
     *
     * writeDir - The name of the directory where the files are found.
     *
     * expiryTime - The period in minutes  added to the file creation time 
     * and compared to the current date in order to check if file is eligible
     * for deletion or not.
     *
     * checkTime - The period of time in minutes the thread remains sleeping 
     * before checking for files eligible for deletion.
     * 
     * fileExtension - The extension of the files to be checked(gif,png, etc).
     */
    public FileCheckThread(String writeDir,int expiryTime,int checkTime,String fileExtension) {
	this.writeDir=writeDir;
	this.expiryTime=expiryTime;
	this.checkTime=checkTime;
	this.fileExtension=fileExtension;
	stop=false;
    }
   
    public void stopThread() {
	stop=true;
    }

    /**
     * Starts checking for files eligible for deletion. The checkTime variable represents the
     * period of time the thread remains sleeping before searching for old files. 
     * The expiryTime variable is used to check a file lifetime. The file name is the time  
     * it was created. The expiry time is a period in miliseconds added to the file creation
     * time and compared to the current time. If result is smaller than the current time, the file
     * is deleted.
     */
    public void run() {

	// Creates a File object to get a list of all files in the 'writeDir' directory.
	File fileList=new File(writeDir);

	// Converts expiry time and check time parameters to miliseconds.
	long fileExpiryTime=60000*expiryTime;
	long fileCheckTime=60000*checkTime;
       
	while (!stop) {

	    try {
		// Thread sleeps for the check time.
		Thread.sleep(fileCheckTime);
	    }
	    catch (InterruptedException e) {
		e.printStackTrace();
	    }

	    // Thread awakes and the list of files is retrieved using a file name filter.
	    // See the class FileFilter.
	    String[] files=fileList.list(new FileFilter());

	    // If files were found...
	    if (files!=null) {

		// Starts a loop to check each file lastModified property.
		for (int counter=0;counter<files.length;counter++) {
		    // Gets each file name and creates a File object to reference file.
		    File fileName=new File(writeDir+files[counter]);
		    
		    // Gets the lastModifed property and the current date in miliseconds.
		    long lastModified=fileName.lastModified();
		    long now=new Date().getTime();

		    /**
		     * Compares the current time with the file creation time plus the expiry time. 
		     * If result is larger the file is deleted.
		     */
		    if (now>fileExpiryTime+lastModified) {
			fileName.delete();
		    }
		    
		}

	    }
	}
    }
    
    // A file name filter.
    class FileFilter implements FilenameFilter {
	public boolean accept(File f,String name) {
	    return (name.endsWith("."+FileCheckThread.this.fileExtension));
	}
    } 


} 
